require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/finman/financialtransaction/financial_transaction_get_transaction_list'
require 'uu/finman/financialtransaction/financial_transaction_attributes'

# Module Unicorn Universe
module UU
  # Module Finman
  module Finman
    # Module of finman Financial transaction
    module FinancialTransaction
      extend self

      # Service path
      PATH = 'uu/finman/financialtransaction/FinancialTransaction'
      # Retrieves a result list of financial transactions for specified business case by UESURI.
      # The resulting attachment list is filtered according to the criteria provided by the criteria attribute.<br>
      #
      # Allowed values for attribute <code>businessCaseSide</code> in query are <code>CREDIT</code> for filtering by credit business case side and <code>DEBIT</code> for filtering by debit business case side. Use it for filtering transaction only by CREDIT / DEBIT side of Business Case(same as main entity). If parameter not used, returns transaction with CREDIT or DEBIT side.
      #
      # @param [String, UU::OS::UESURI] business_case_uri UESURI of the business case.
      # @param [UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList] criteria DTO Financial_transaction_get_transaction_list containing paging and filtering criteria. Available parameters are:<br> {FinancialTransactionAttributes#creation_time creationTime}, {FinancialTransactionAttributes#credit_vat_date creditVatDate}, {FinancialTransactionAttributes#debit_vat_date debitVatDate}, {FinancialTransactionAttributes#meta_artifact_uri metaArtifactUri}, {FinancialTransactionAttributes#pay_date payDate}, {FinancialTransactionAttributes#state_name stateName}, {FinancialTransactionAttributes#state_type stateType}, businessCaseSide.
      # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there.
      #
      # @return [UU::OS::REST::ResultList<UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList, UU::Finman::FinancialTransaction::FinancialTransactionAttributes>] List of financial transaction.
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to get Financial Transaction list.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # Get transaction list on  specific Business Case
      #      UU::Finman::FinancialTransaction.get_transaction_list(business_case_uri,
      #         :query => "creationTime > '2015-01-01' AND metaArtifactUri = 'ues:TERRITORY:META_ARTIFACT_OF_FINANCIAL_TRANSACTION' AND stateType = ACTIVE");
      def get_transaction_list(bc_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        dto = UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getTransactionList', bc_uri)
          return UU::OS::REST::ResultList.new(UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList, UU::Finman::FinancialTransaction::FinancialTransactionAttributes, res)
        end
      end

    end
  end
end
